//
// demo for the pdb2vrml.
//
// Copyright 2001 by Vieri Di Paola.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "gnubioq@yahoo.com".
//

#include <iostream>
#include <stdlib.h>
#include <pdb2vrml.h>


int main(int argc, char **argv) {
  
  // Greet...
  cout << "\n * PDB2VRML demo program\n * May 2000\n * GNU General Public License\n * Vieri Di Paola\n * gnubioq@yahoo.com\n * vieridipaola@yahoo.com\n * http://www.geocities.com/vieridipaola\n * http://www.geocities.com/gnubioq\n * http://www.cli.di.unipi.it/~dipaola\n\n";
  
  // Get upset...
  if (argc<4) {
    cout << " USAGE:\n\t" << argv[0] << " pdbfile vrmlfile displaymode\n\n\tdisplaymode = 1 --> SPACEFILL\n\t\t      2 --> BALLANDSTICK\n\t\t      3 --> WIREFRAME\n\t\t      4 --> SURFACE\n\nThis is only a demo. Check the documentation for more information.\n\n";
    return 1;
    }
  
  // Create converter and load pdb file.
  pdb2vrml converter(argv[1]);
  // Alternatively, do:
  // pdb2vrml converter;
  // converter.pdb(argv[1]);
  
  // Set background textures. 
  // converter.setTopTexture("textures/sky.png");
  // converter.setBottomTexture("textures/ground.png");
  // converter.setFrontTexture("textures/mountains.png"); etc...

  // Set background colors. 
  // converter.SKY1_COLOR_RED = 0.3; converter.SKY1_COLOR_GREEN = 0.4; converter.SKY1_COLOR_BLUE = 0.6;
  // converter.SKY_ANGLE1 = 0.4; converter.SKY_ANGLE2 = 0.9; etc...
  // converter.GROUND1_COLOR_RED = 0.1; converter.GROUND1_COLOR_GREEN = 0.2; converter.GROUND1_COLOR_BLUE = 0.8;

  // Make sure to allow background rendering. 
  converter.showBackground();

  // Set atom colors. 
  // converter.C_COLOR_RED = 0.3; converter.C_COLOR_GREEN = 0.4; converter.C_COLOR_BLUE = 0.6;
  // etc...
    
  // Set textures. Overrides default colors unless texture not found. Not yet available from command line.
  converter.setCarbonTexture("textures/carbon.png");
  converter.setNitrogenTexture("textures/nitrogen.png");
  converter.setHydrogenTexture("textures/hydrogen.png");
  converter.setPhosphateTexture("textures/phosphate.png");
  converter.setSulfurTexture("textures/sulfur.png");
  converter.setOxygenTexture("textures/oxygen.png");
  converter.setNBondTexture("textures/nbond.png");
  converter.setCBondTexture("textures/cbond.png");
  converter.setHBondTexture("textures/hbond.png");
  converter.setSBondTexture("textures/sbond.png");
  // If converter called again on other pdb file, texture can be eliminated like so:
  // converter.setCarbonTexture(NULL);

  // Convert to vrml and compress (default: 9).
  converter.vrml(argv[2], atoi(argv[3]));
  // Alternatively, do:
  // converter.vrml(argv[2], atoi(argv[3]), 0);
  // where 0 = no compression and 9 = best compression
  
  // Check for errors.
  switch (converter.status) {
    case PV_MEM_ERR   : cout << "ERROR: Not enough memory." << endl;
                        break;
    case PV_IF_ERR    : cout << "ERROR: Trouble with input file." << endl;
                        break;
    case PV_OF_ERR    : cout << "ERROR: Trouble with output file." << endl;
                        break;
    case PV_PDB_ERR   : cout << "ERROR: Invalid PDB format." << endl;
                        break;
    case PV_PDBII_ERR : cout << "ERROR: Invalid PDB records." << endl;
                        break;
    default           : cout << "Successful conversion." << endl;
                        break;
  }
  
  // Bye.
  cout << "Please report any bugs or improvements to:\n\tgnubioq@yahoo.com or\n\tvieridipaola@yahoo.com\n";

  return 0;
}
