//
// pdb2vrml
//
// Copyright 2001 by Vieri Di Paola.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "gnubioq@yahoo.com".
//


#ifndef PDB2VRML_H
#define PDB2VRML_H

#include "AtomInfo.h"
#include "MolInfo.h"
#include <zlib.h>
#include <strstream>
#include <fstream>

class pdb2vrml {

public:

  pdb2vrml(const char* pdbfilename = 0L);
  ~pdb2vrml(void);
  
  MolInfo *Mol;  // use a "friend" class instead
  
  int pdb(const char*);
  int vrml(const char*, int, const char *compr = "w9"); // second param is one of the DISPLAY_...
  int writevrml(int);                                   // param is one of the DISPLAY_...
  void quit(void);                                      // force destruction
  
  void setCarbonTexture(const char*);
  void setNitrogenTexture(const char*);
  void setHydrogenTexture(const char*);
  void setPhosphateTexture(const char*);
  void setSulfurTexture(const char*);
  void setOxygenTexture(const char*);
  void setNBondTexture(const char*);
  void setCBondTexture(const char*);
  void setHBondTexture(const char*);
  void setSBondTexture(const char*);

  int status;

  float C_COLOR_RED, C_COLOR_GREEN, C_COLOR_BLUE;
  float O_COLOR_RED, O_COLOR_GREEN, O_COLOR_BLUE;    
  float N_COLOR_RED, N_COLOR_GREEN, N_COLOR_BLUE;   
  float P_COLOR_RED, P_COLOR_GREEN, P_COLOR_BLUE;    
  float H_COLOR_RED, H_COLOR_GREEN, H_COLOR_BLUE;   
  float S_COLOR_RED, S_COLOR_GREEN, S_COLOR_BLUE;    
  float NB_COLOR_RED, NB_COLOR_GREEN, NB_COLOR_BLUE;   
  float CB_COLOR_RED, CB_COLOR_GREEN, CB_COLOR_BLUE;  
  float HB_COLOR_RED, HB_COLOR_GREEN, HB_COLOR_BLUE;
  float SB_COLOR_RED, SB_COLOR_GREEN, SB_COLOR_BLUE; 

  void showBackground(void);
  void hideBackground(void);
  void setTopTexture(const char*);
  void setBottomTexture(const char*);
  void setFrontTexture(const char*);
  void setBackTexture(const char*);
  void setRightTexture(const char*);
  void setLeftTexture(const char*);
  float SKY1_COLOR_RED, SKY1_COLOR_GREEN, SKY1_COLOR_BLUE;
  float SKY2_COLOR_RED, SKY2_COLOR_GREEN, SKY2_COLOR_BLUE;
  float SKY3_COLOR_RED, SKY3_COLOR_GREEN, SKY3_COLOR_BLUE;
  float SKY4_COLOR_RED, SKY4_COLOR_GREEN, SKY4_COLOR_BLUE;
  float SKY5_COLOR_RED, SKY5_COLOR_GREEN, SKY5_COLOR_BLUE;
  float SKY6_COLOR_RED, SKY6_COLOR_GREEN, SKY6_COLOR_BLUE;
  float SKY_ANGLE1, SKY_ANGLE2, SKY_ANGLE3, SKY_ANGLE4, SKY_ANGLE5;
  float GROUND1_COLOR_RED, GROUND1_COLOR_GREEN, GROUND1_COLOR_BLUE;
  float GROUND2_COLOR_RED, GROUND2_COLOR_GREEN, GROUND2_COLOR_BLUE;
  float GROUND3_COLOR_RED, GROUND3_COLOR_GREEN, GROUND3_COLOR_BLUE;
  float GROUND_ANGLE1, GROUND_ANGLE2;
 
private:

//  gzFile vrmlfile;
  void traceBond(gzFile, int, int, float, float, float, unsigned char, unsigned char);
  void traceSphere(gzFile, const char*, float, float, float, const char*);
  float BONDheight;
  void WireHeader(gzFile, float, float, float);
  void SurfaceHeader(gzFile, const char*, float, float, float);
//  void traceWire(int, char, int, const char*);
  void traceWire(void);
  void traceSurface(void);
  void traceBallStick(void);
  void traceSpaceFill(void);
  const char *TextureExists(const char*);
  int bsi;
  int bkScene;
  const char *ctex, *ntex, *otex, *htex, *ptex, *stex, *nbtex, *hbtex, *cbtex, *sbtex;
  const char *toptex, *bottomtex, *fronttex, *backtex, *righttex, *lefttex;
};

#endif

