//
// pdb2vrml
//
// Copyright 2001 by Vieri Di Paola.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "gnubioq@yahoo.com".
//


#include <MolInfo.h>

MolInfo::MolInfo(void) {
  status = 0;
  done = 0;
  AtomList = 0L;
  MaxX = MaxY = MaxZ = MinX = MinY = 0;
}

void MolInfo::setpdb(const char *pdbfname) {
  pdbfile = gzopen(pdbfname, "r");
  if (pdbfile == Z_NULL) { status = PV_IF_ERR; return; }
}

void MolInfo::setvrml(const char *vrmlfname, const char *compr) {
  vrmlfile = gzopen(vrmlfname, compr);
  if (vrmlfile == Z_NULL) { status = PV_OF_ERR; return; }
  // try w0-w9 for least-best compression...
}

MolInfo::~MolInfo(void) {
  if (AtomList) delete[](AtomList);
  gzclose(pdbfile);
  gzclose(vrmlfile);
}

int MolInfo::initAtomList(int tota) {
  void *EMEM = malloc(104 * (tota+1));
  if ( 0 == EMEM ) 
     return -1;
  else 
  AtomList = new AtomInfo[tota]();
}

// int MolInfo::getAtomList(int tota) {
int MolInfo::getAtomList(void) {
  char numstr[9];
  int apos = 0;
  char resid[5];

  char buf[150];

  gzrewind(pdbfile);
  while (!gzeof(pdbfile)) {
    gzgets(pdbfile,buf,150);
    if (!strcmp(buf, "END") || 
	!strncmp(buf, "END ", 4) || 
	!strncmp(buf, "ENDMDL", 6)) break;
    if (!strncmp(buf, "ATOM  ", 6) || !strncmp(buf, "HETATM", 6))
       {
       memset(numstr, 0, 9 * sizeof(char));
       strncpy(numstr, buf + 30, 8);
       AtomList[apos].x = (float) atof(numstr);
       if (AtomList[apos].x > MaxX) MaxX = AtomList[apos].x;
       if (AtomList[apos].x < MinX) MinX = AtomList[apos].x;
       memset(numstr, 0, 9 * sizeof(char));
       strncpy(numstr, buf + 38, 8);
       AtomList[apos].y = (float) atof(numstr);
       if (AtomList[apos].y > MaxY) MaxY = AtomList[apos].y;
       if (AtomList[apos].y < MinY) MinY = AtomList[apos].y;
       memset(numstr, 0, 9 * sizeof(char));
       strncpy(numstr, buf + 46, 8);
       AtomList[apos].z = (float) atof(numstr);
       if (AtomList[apos].z > MaxZ) MaxZ = AtomList[apos].z;
       strncpy(AtomList[apos].name,buf + 12, 2);
       strncpy(AtomList[apos].resname,buf + 17, 2);
       strncpy(resid,buf + 22, 4);
       AtomList[apos].resid = atoi(resid);
       apos++;
       }
    } // while
  return 0;
}

void MolInfo::getCONECTbonds(unsigned char Btype, char *numstrptr, char *bufptr, int conAtomV, int atmpV, int itmpptr) {
  int ctmp, iitmp;
         memset(numstrptr, 0, 5 * sizeof(char));
         strncpy(numstrptr, bufptr + conAtomV, 4);
         ctmp = atoi(numstrptr);
         ctmp--; // because bond arrays start at 0, not 1.
         if ( (itmpptr<MAXBONDS) && (ctmp>=0) && ((int)strlen(bufptr)>=conAtomV+4) ) 
            { 
	    if (ctmp>atmpV) {
	       AtomList[atmpV].bonds[Btype][itmpptr] = ctmp; 
	       itmpptr++;
	       } else {
	       for (iitmp=0; iitmp<MAXBONDS; iitmp++) 
	           if (AtomList[ctmp].bonds[Btype][iitmp] == -1) break;
	       AtomList[ctmp].bonds[Btype][iitmp] = atmpV;
	       }
	    }
}

void MolInfo::getAtomBonds(void) {
  float dist,tmp; int atmp,itmp;
  float m1r = 0.0;
  float m2r = 0.0;
  for (unsigned int m1=1-1; m1<=TotAtoms-1; m1++) {
      for (unsigned int m2=m1+1; m2<=TotAtoms-1; m2++) {
      dist = sqrt(
             (pow((AtomList[m1].x - AtomList[m2].x),2)) +
             (pow((AtomList[m1].y - AtomList[m2].y),2)) +
             (pow((AtomList[m1].z - AtomList[m2].z),2)) 
             );
      switch (AtomList[m1].name[1]) {
       case 'C' : m1r = C_ATOM_RADIUS; break;
       case 'N' : m1r = N_ATOM_RADIUS; break;
       case 'O' : m1r = O_ATOM_RADIUS; break;
       case 'P' : m1r = P_ATOM_RADIUS; break;
       case 'S' : m1r = S_ATOM_RADIUS; break;
      }
      switch (AtomList[m2].name[1]) {
       case 'C' : m2r = C_ATOM_RADIUS; break;
       case 'N' : m2r = N_ATOM_RADIUS; break;
       case 'O' : m2r = O_ATOM_RADIUS; break;
       case 'P' : m2r = P_ATOM_RADIUS; break;
       case 'S' : m2r = S_ATOM_RADIUS; break;
      }
      tmp = fabs(m1r + m2r);
      if ( (dist <= tmp)
         && (m2 > m1)
         && (  (AtomList[m1].name[1]!='H')
            && (AtomList[m2].name[1]!='H') ) )
         {
	 itmp = 0;
	 while ((AtomList[m1].bonds[0][itmp] > -1) && (itmp<MAXBONDS)) itmp++;
	 AtomList[m1].bonds[0][itmp] = m2;
	 }
      } // second for loop
  } // first for loop
  // Now let's look for CONECT records...
  char buf[150];
  char numstr[5];
  gzrewind(pdbfile);
  while (!gzeof(pdbfile)) {
    gzgets(pdbfile,buf,150);
    if (!strncmp(buf, "CONECT", 6))
       {
       memset(numstr, 0, 5 * sizeof(char));
       strncpy(numstr, buf + 7, 4);
       atmp = atoi(numstr);
       atmp--; // because bond arrays start at 0, not 1.
       itmp = 0;
       while ((AtomList[atmp].bonds[1][itmp] > -1) && (itmp<MAXBONDS)) itmp++;

       // get bonded atoms 2,3,4,5
       for (int conAtom=12; conAtom<=27; conAtom+=5) 
         getCONECTbonds(1, numstr, buf, conAtom, atmp, itmp);
	 
       // get bonded hydrogen atoms 6,7,..,9,10
       // 6,7:
       itmp = 0;
       while ((AtomList[atmp].bonds[2][itmp] > -1) && (itmp<MAXBONDS)) itmp++;
       for (int conAtom=32; conAtom<=37; conAtom+=5) 
         getCONECTbonds(2, numstr, buf, conAtom, atmp, itmp);
	 
       // 9,10:
       for (int conAtom=47; conAtom<=52; conAtom+=5) 
         getCONECTbonds(2, numstr, buf, conAtom, atmp, itmp);
	        
       // get salt bridges 8,..,11
       // 8:
       itmp = 0;
       while ((AtomList[atmp].bonds[3][itmp] > -1) && (itmp<MAXBONDS)) itmp++;
         getCONECTbonds(3, numstr, buf, 42, atmp, itmp);
	 
       // 11:
       getCONECTbonds(3, numstr, buf, 57, atmp, itmp);

       } // if "CONECT"
    } // while
  
}

unsigned int MolInfo::getTotAtoms(void) {
  char buf[150];
  unsigned int ta = 0;
  gzrewind(pdbfile);
  while (!gzeof(pdbfile)) {
    gzgets(pdbfile,buf,150);
    if (!strcmp(buf, "END") || 
	!strncmp(buf, "END ", 4) || 
	!strncmp(buf, "ENDMDL", 6)) break;
    if (!strncmp(buf, "ATOM  ", 6) || !strncmp(buf, "HETATM", 6))
       ta++;
    }
  return ta;
}

