//
// pdb2vrml
//
// Copyright 2001 by Vieri Di Paola.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "gnubioq@yahoo.com".
//


#include <pdb2vrml.h>
#include <iostream>
#include <fstream>
#include <math.h>

#include <stdio.h>

pdb2vrml::pdb2vrml(const char *pdbfilename) {
  bkScene = 0;
  C_COLOR_RED = 0.3;
  C_COLOR_GREEN = 0.3;
  C_COLOR_BLUE = 0.3;
  O_COLOR_RED = 0.9;
  O_COLOR_GREEN = 0.1;
  O_COLOR_BLUE = 0.1;
  N_COLOR_RED = 0.1;
  N_COLOR_GREEN = 0.9;
  N_COLOR_BLUE = 0.1;
  P_COLOR_RED = 0.1;
  P_COLOR_GREEN = 0.1;
  P_COLOR_BLUE = 0.9;
  H_COLOR_RED = 1.0;
  H_COLOR_GREEN = 1.0;
  H_COLOR_BLUE = 1.0;
  S_COLOR_RED = 0.8;
  S_COLOR_GREEN = 0.8;
  S_COLOR_BLUE = 0.1;
  NB_COLOR_RED = 0.4;
  NB_COLOR_GREEN = 0.4;
  NB_COLOR_BLUE = 0.4;
  CB_COLOR_RED = 0.5;
  CB_COLOR_GREEN = 0.5;
  CB_COLOR_BLUE = 0.5;
  HB_COLOR_RED = 0.6;
  HB_COLOR_GREEN = 0.6;
  HB_COLOR_BLUE = 0.6;
  SB_COLOR_RED = 0.7;
  SB_COLOR_GREEN = 0.7;
  SB_COLOR_BLUE = 0.7;
  status = 0;
  BONDheight = 0.0;
  ctex = ntex = otex = htex = ptex = stex = nbtex = hbtex = sbtex = cbtex = 0L;
  toptex = bottomtex = fronttex = backtex = righttex = lefttex = 0L;
  Mol = new MolInfo();
  if ( (status = Mol->status) < 0 ) return;
  if (pdbfilename) {
    status = pdb(pdbfilename);
    if (status == 0) 
       cout << "DEBUG: PDB file successfully loaded.\n";
       else {
       cout << "DEBUG: PDB NOT file successfully loaded.\n";
//     status = -1;
       return;
       }
  }
}

pdb2vrml::~pdb2vrml(void) {
  if (Mol) delete(Mol);
}

void pdb2vrml::quit(void) {
  this->~pdb2vrml();
}

void pdb2vrml::showBackground(void) {
  bkScene = 1;
}

void pdb2vrml::hideBackground(void) {
  bkScene = 0;
}

int pdb2vrml::vrml(const char *vrmlfilename, int dispmode, const char *compr) {
  Mol->setvrml(vrmlfilename, compr);
  writevrml(dispmode);
}

int pdb2vrml::pdb(const char *pdbfilename) {
  Mol->setpdb(pdbfilename);
  if ( (Mol->TotAtoms = Mol->getTotAtoms()) <= 0 ) { 
//     if (Mol) delete(Mol);
     return PV_PDB_ERR; 
     }
  cout << "DEBUG: Total atoms = " << Mol->TotAtoms << endl;
  
  if ( Mol->initAtomList(Mol->TotAtoms) < 0 ) 
  return PV_MEM_ERR;

  if ( Mol->getAtomList() < 0 ) {
  cout << "DEBUG: Error getting atom list." << endl;
  return PV_PDBII_ERR;
  }
  else cout << "DEBUG: Got atom list." << endl;

  Mol->getAtomBonds();
    
  return 0;
}

const char *pdb2vrml::TextureExists(const char *T) {
  ifstream Tin(T);
  if (!Tin.is_open())
    return 0L;
  else return T;
}

void pdb2vrml::setTopTexture(const char *tfname) {
  toptex = TextureExists(tfname);
}

void pdb2vrml::setBottomTexture(const char *tfname) {
  bottomtex = TextureExists(tfname);
}

void pdb2vrml::setFrontTexture(const char *tfname) {
  fronttex = TextureExists(tfname);
}

void pdb2vrml::setBackTexture(const char *tfname) {
  backtex = TextureExists(tfname);
}

void pdb2vrml::setLeftTexture(const char *tfname) {
  lefttex = TextureExists(tfname);
}

void pdb2vrml::setRightTexture(const char *tfname) {
  righttex = TextureExists(tfname);
}

void pdb2vrml::setCarbonTexture(const char *tfname) {
  ctex = TextureExists(tfname);
}

void pdb2vrml::setNitrogenTexture(const char *tfname) {
  ntex = TextureExists(tfname);
}

void pdb2vrml::setHydrogenTexture(const char *tfname) {
  htex = TextureExists(tfname);
}

void pdb2vrml::setPhosphateTexture(const char *tfname) {
  ptex = TextureExists(tfname);
}

void pdb2vrml::setSulfurTexture(const char *tfname) {
  stex = TextureExists(tfname);
}

void pdb2vrml::setOxygenTexture(const char *tfname) {
  otex = TextureExists(tfname);
}

void pdb2vrml::setNBondTexture(const char *tfname) {
  nbtex = TextureExists(tfname);
}

void pdb2vrml::setCBondTexture(const char *tfname) {
  cbtex = TextureExists(tfname);
}

void pdb2vrml::setHBondTexture(const char *tfname) {
  hbtex = TextureExists(tfname);
}

void pdb2vrml::setSBondTexture(const char *tfname) {
  sbtex = TextureExists(tfname);
}


void pdb2vrml::traceBond(gzFile vrmlfileptr, int tbi, int tbitmp, float col1, float col2, float col3, unsigned char mandatory, unsigned char remember) {
  float mtmp, newX, newY, newZ, Zvect;
               gzprintf(vrmlfileptr, "Transform {\n\tchildren [\n");
               mtmp = sqrt(pow((Mol->AtomList[tbi].x - Mol->AtomList[tbitmp].x), 2)
                          +pow((Mol->AtomList[tbi].y - Mol->AtomList[tbitmp].y), 2)
                          +pow((Mol->AtomList[tbi].z - Mol->AtomList[tbitmp].z), 2)
                          );
	       if ( (BONDheight >= (mtmp - 0.8 + 0.05)) && 
	            (BONDheight <= (mtmp + 0.8 - 0.05)) &&
		    (mandatory == 0) )
                    gzprintf(vrmlfileptr, "\t\tUSE BOND\n");
	       else {
                 if (remember == 0)
                    gzprintf(vrmlfileptr, "\t\tShape {\n");
	         else {
	         gzprintf(vrmlfileptr, "\t\tDEF BOND Shape {\n");
	         BONDheight = mtmp;
	         }
		    
               gzprintf(vrmlfileptr, "\t\t\tappearance Appearance {\n");
               if ( (nbtex) && ( (col1 == NB_COLOR_RED) && (col2 == NB_COLOR_GREEN) && (col3 == NB_COLOR_BLUE) ) ) {
                 gzprintf(vrmlfileptr, "\t\t\t\ttexture ImageTexture {\n");
                 gzprintf(vrmlfileptr, "\t\t\t\turl \"%s\"\n", nbtex);
                 } else 
               if ( (hbtex) && ( (col1 == HB_COLOR_RED) && (col2 == HB_COLOR_GREEN) && (col3 == HB_COLOR_BLUE) ) ) {  
                 gzprintf(vrmlfileptr, "\t\t\t\ttexture ImageTexture {\n");
                 gzprintf(vrmlfileptr, "\t\t\t\turl \"%s\"\n", hbtex);
                 } else 
               if ( (sbtex) && ( (col1 == SB_COLOR_RED) && (col2 == SB_COLOR_GREEN) && (col3 == SB_COLOR_BLUE) ) ) {      
                 gzprintf(vrmlfileptr, "\t\t\t\ttexture ImageTexture {\n");
                 gzprintf(vrmlfileptr, "\t\t\t\turl \"%s\"\n", sbtex);
                 } else 
               if ( (cbtex) && ( (col1 == CB_COLOR_RED) && (col2 == CB_COLOR_GREEN) && (col3 == CB_COLOR_BLUE) ) ) {      
                 gzprintf(vrmlfileptr, "\t\t\t\ttexture ImageTexture {\n");
                 gzprintf(vrmlfileptr, "\t\t\t\turl \"%s\"\n", cbtex);
                 } else {
                 gzprintf(vrmlfileptr, "\t\t\t\tmaterial Material {\n");
                 gzprintf(vrmlfileptr, "\t\t\t\tdiffuseColor %f %f %f\n", col1, col2, col3);
                 }
               gzprintf(vrmlfileptr, "\t\t\t\t}\n");
               gzprintf(vrmlfileptr, "\t\t\t}\n");
               gzprintf(vrmlfileptr, "\t\t\tgeometry Cylinder {\n");
               gzprintf(vrmlfileptr, "\t\t\t\ttop    FALSE\n");
               gzprintf(vrmlfileptr, "\t\t\t\tbottom FALSE\n");
               gzprintf(vrmlfileptr, "\t\t\t\tradius 0.100000\n");


               gzprintf(vrmlfileptr, "\t\t\t\theight %f\n", mtmp);
               gzprintf(vrmlfileptr, "\t\t\t}\n\t\t}\n");
	       } // first if USE BOND
               mtmp = ((Mol->AtomList[tbi].x + Mol->AtomList[tbitmp].x) / 2);
               newX = Mol->AtomList[tbi].x - mtmp;
               gzprintf(vrmlfileptr, "\t]\n\ttranslation\t%f ", mtmp);
               mtmp = ((Mol->AtomList[tbi].y + Mol->AtomList[tbitmp].y) / 2);
               newY = Mol->AtomList[tbi].y - mtmp;
               gzprintf(vrmlfileptr, "%f ", mtmp);
               mtmp = ((Mol->AtomList[tbi].z + Mol->AtomList[tbitmp].z) / 2);
               newZ = Mol->AtomList[tbi].z - mtmp;
               gzprintf(vrmlfileptr, "%f\n\n", mtmp);
               if (newX>0) {
                  mtmp = ((Mol->AtomList[tbi].x + Mol->AtomList[tbitmp].x) / 2);
                  newX = Mol->AtomList[tbitmp].x - mtmp;
                  mtmp = ((Mol->AtomList[tbi].y + Mol->AtomList[tbitmp].y) / 2);
                  newY = Mol->AtomList[tbitmp].y - mtmp;
                  mtmp = ((Mol->AtomList[tbi].z + Mol->AtomList[tbitmp].z) / 2);
                  newZ = Mol->AtomList[tbitmp].z - mtmp;
                  }
               gzprintf(vrmlfileptr, "\trotation\t");
               if (newZ == 0) {
	          gzprintf(vrmlfileptr, "0.0 0.0 1.0 ");
		  }
		  else 
  		  if (newX == 0) {
                     gzprintf(vrmlfileptr, "1.0 0.0 0.0 ");
  		     }
		     else
		     if (newY == 0) {
		        gzprintf(vrmlfileptr, "0.0 1.0 0.0 ");
			}
		        else {
			     if ((newX<0) && (newZ<0)) {
                                Zvect = newX / newZ;
                                gzprintf(vrmlfileptr, "-1.0 0.0 %f ", Zvect);
                                }
                                else 
                                {
		      	        Zvect = ( newX * -1 ) / newZ;
                                gzprintf(vrmlfileptr, "1.0 0.0 %f ", Zvect);
                                }
			     }
               if (newY == 0) newY = 0.000000001;
	       mtmp = atan(
                      sqrt (pow(newX,2) + pow(newZ,2))
                      / fabs(newY)
                      );
               if (newY<0) mtmp = M_PI_2 + (M_PI_2 - mtmp);
               gzprintf(vrmlfileptr, "%f\n}\n", mtmp);
}

void pdb2vrml::traceSphere(gzFile vrmlfileptr, const char *Atype, float Ctype1, float Ctype2, float Ctype3, const char *Rtype) {
                       gzprintf(vrmlfileptr, "\t\tDEF %s Shape {\n", Atype);
                       gzprintf(vrmlfileptr, "\t\t\tappearance Appearance {\n");
                       if ( (Atype == "C") && (ctex) ) {
                       gzprintf(vrmlfileptr, "\t\t\t\ttexture ImageTexture {\n");
                       gzprintf(vrmlfileptr, "\t\t\t\turl \"%s\"\n", ctex);
                       } else 
                       if ( (Atype == "O") && (otex) ) {
                       gzprintf(vrmlfileptr, "\t\t\t\ttexture ImageTexture {\n");
                       gzprintf(vrmlfileptr, "\t\t\t\turl \"%s\"\n", otex);
                       } else 
                       if ( (Atype == "H") && (htex) ) {
                       gzprintf(vrmlfileptr, "\t\t\t\ttexture ImageTexture {\n");
                       gzprintf(vrmlfileptr, "\t\t\t\turl \"%s\"\n", htex);
                       } else 
                       if ( (Atype == "P") && (ptex) ) {
                       gzprintf(vrmlfileptr, "\t\t\t\ttexture ImageTexture {\n");
                       gzprintf(vrmlfileptr, "\t\t\t\turl \"%s\"\n", ptex);
                       } else 
                       if ( (Atype == "S") && (stex) ) {
                       gzprintf(vrmlfileptr, "\t\t\t\ttexture ImageTexture {\n");
                       gzprintf(vrmlfileptr, "\t\t\t\turl \"%s\"\n", stex);
                       } else 
                       if ( (Atype == "N") && (ntex) ) {
                       gzprintf(vrmlfileptr, "\t\t\t\ttexture ImageTexture {\n");
                       gzprintf(vrmlfileptr, "\t\t\t\turl \"%s\"\n", ntex);
                       } else {
                       gzprintf(vrmlfileptr, "\t\t\t\tmaterial Material {\n");
                       gzprintf(vrmlfileptr, "\t\t\t\tdiffuseColor %f %f %f\n", Ctype1, Ctype2, Ctype3);
                       }
                       gzprintf(vrmlfileptr, "\t\t\t\t}\n");
                       gzprintf(vrmlfileptr, "\t\t\t}\n");
                       gzprintf(vrmlfileptr, "\t\t\tgeometry Sphere {\n");
                       gzprintf(vrmlfileptr, "\t\t\t\tradius    %s\n", Rtype);
                       gzprintf(vrmlfileptr, "\t\t\t}\n\t\t}\n");
}

void pdb2vrml::WireHeader(gzFile vrmlfileptr, float Ctype1, float Ctype2, float Ctype3) {
               gzprintf(vrmlfileptr, "Shape {\n");
               gzprintf(vrmlfileptr, "\tappearance Appearance {\n");
               gzprintf(vrmlfileptr, "\t\tmaterial Material {\n");
               gzprintf(vrmlfileptr, "\t\t\temissiveColor %f %f %f\n", Ctype1, Ctype2, Ctype3);
               gzprintf(vrmlfileptr, "\t\t}\n");
               gzprintf(vrmlfileptr, "\t}\n");
               gzprintf(vrmlfileptr, "\tgeometry IndexedLineSet {\n");
               gzprintf(vrmlfileptr, "\t\tcoord Coordinate {\n");
               gzprintf(vrmlfileptr, "\t\t\tpoint [\n");
}

void pdb2vrml::SurfaceHeader(gzFile vrmlfileptr, const char *Atype, float Ctype1, float Ctype2, float Ctype3) {
               gzprintf(vrmlfileptr, "Shape {\n");
               gzprintf(vrmlfileptr, "\tappearance Appearance {\n");
                       if ( (Atype == "C") && (ctex) ) {
                       gzprintf(vrmlfileptr, "\t\ttexture ImageTexture {\n");
                       gzprintf(vrmlfileptr, "\t\t\turl \"%s\"\n", ctex);
                       } else 
                       if ( (Atype == "O") && (otex) ) {
                       gzprintf(vrmlfileptr, "\t\ttexture ImageTexture {\n");
                       gzprintf(vrmlfileptr, "\t\t\turl \"%s\"\n", otex);
                       } else 
                       if ( (Atype == "H") && (htex) ) {
                       gzprintf(vrmlfileptr, "\t\ttexture ImageTexture {\n");
                       gzprintf(vrmlfileptr, "\t\t\turl \"%s\"\n", htex);
                       } else 
                       if ( (Atype == "P") && (ptex) ) {
                       gzprintf(vrmlfileptr, "\t\ttexture ImageTexture {\n");
                       gzprintf(vrmlfileptr, "\t\t\turl \"%s\"\n", ptex);
                       } else 
                       if ( (Atype == "S") && (stex) ) {
                       gzprintf(vrmlfileptr, "\t\ttexture ImageTexture {\n");
                       gzprintf(vrmlfileptr, "\t\t\turl \"%s\"\n", stex);
                       } else 
                       if ( (Atype == "N") && (ntex) ) {
                       gzprintf(vrmlfileptr, "\t\ttexture ImageTexture {\n");
                       gzprintf(vrmlfileptr, "\t\t\turl \"%s\"\n", ntex);
                       } else {
                       gzprintf(vrmlfileptr, "\t\tmaterial Material {\n");
                       gzprintf(vrmlfileptr, "\t\t\tdiffuseColor %f %f %f\n", Ctype1, Ctype2, Ctype3);
                       }
               gzprintf(vrmlfileptr, "\t\t}\n");
               gzprintf(vrmlfileptr, "\t}\n");
               gzprintf(vrmlfileptr, "\tgeometry IndexedFaceSet {\n");
               gzprintf(vrmlfileptr, "\t\tcoord Coordinate {\n");
               gzprintf(vrmlfileptr, "\t\t\tpoint [\n");
}

void pdb2vrml::traceWire(void) {
  int bi;
  unsigned int posA, v;
  ostrstream bstream[9];
  unsigned int coordiptr[9];
  

  // init...
  for (v = 0; v < 9; v++) coordiptr[v] = 0;
  
  for (posA = 0; posA < Mol->TotAtoms; posA++) {
    for (int B = 0; B <= 4; B++) {
      for (bi = 0; bi < MAXBONDS; bi++) {
        if (Mol->AtomList[posA].bonds[B][bi] > -1) {
          switch(Mol->AtomList[posA].name[1]) {
	    case 'C' : bsi = 0;
	 	       break;
            case 'N' : bsi = 1;
		       break;
            case 'O' : bsi = 2;
		       break;
            case 'H' : bsi = 3;
                       break;
	    case 'S' : bsi = 4;
		       break;
	    case 'P' : bsi = 5;
		       break;
	    } // switch
	  if ( (Mol->AtomList[posA].name[1] == Mol->AtomList[Mol->AtomList[posA].bonds[B][bi]].name[1])
	     || ( (B >= 1) && (B <= 3) ) ) {
	       if ( (B >= 1) && (B <= 3) ) bsi = B + 5;
               bstream[bsi] << "\t\t\t" << Mol->AtomList[posA].x << " " << Mol->AtomList[posA].y << " " << Mol->AtomList[posA].z << " ,\n\t\t\t" << Mol->AtomList[Mol->AtomList[posA].bonds[B][bi]].x << " " << Mol->AtomList[Mol->AtomList[posA].bonds[B][bi]].y << " " << Mol->AtomList[Mol->AtomList[posA].bonds[B][bi]].z << " ,\n";
	  } else {
               bstream[bsi] << "\t\t\t" << Mol->AtomList[posA].x << " " << Mol->AtomList[posA].y << " " << Mol->AtomList[posA].z << " ,\n\t\t\t" << ((Mol->AtomList[posA].x + Mol->AtomList[Mol->AtomList[posA].bonds[B][bi]].x) / 2) << " " << ((Mol->AtomList[posA].y + Mol->AtomList[Mol->AtomList[posA].bonds[B][bi]].y) / 2) << " " << ((Mol->AtomList[posA].z + Mol->AtomList[Mol->AtomList[posA].bonds[B][bi]].z) / 2) << " ,\n";
               for (v = 0; v < MAXBONDS; v++)
	         if (Mol->AtomList[Mol->AtomList[posA].bonds[B][bi]].bonds[4][v] == -1) 
	            break;
               Mol->AtomList[Mol->AtomList[posA].bonds[B][bi]].bonds[4][v] = posA;
	  }
          coordiptr[bsi]++;
	} // if > -1
      }
    }
  }

  for (v = 0; v < 9; v++) {
    bstream[v] << ends;
    if (coordiptr[v] > 0) {
      switch (v) {
        case 0 : WireHeader(Mol->vrmlfile, C_COLOR_RED, C_COLOR_GREEN, C_COLOR_BLUE); // carbon color
	         break;
        case 1 : WireHeader(Mol->vrmlfile, N_COLOR_RED, N_COLOR_GREEN, N_COLOR_BLUE); // nitrogen color
	         break;
        case 2 : WireHeader(Mol->vrmlfile, O_COLOR_RED, O_COLOR_GREEN, O_COLOR_BLUE); // oxygen
	         break;
        case 3 : WireHeader(Mol->vrmlfile, H_COLOR_RED, H_COLOR_GREEN, H_COLOR_BLUE); // hydrogen
	         break;
        case 4 : WireHeader(Mol->vrmlfile, S_COLOR_RED, S_COLOR_GREEN, S_COLOR_BLUE); // sulfur
	         break;
        case 5 : WireHeader(Mol->vrmlfile, P_COLOR_RED, P_COLOR_GREEN, P_COLOR_BLUE); // phosphate
	         break;
        case 6 : WireHeader(Mol->vrmlfile, CB_COLOR_RED, CB_COLOR_GREEN, CB_COLOR_BLUE); // cbonds
	         break;
        case 7 : WireHeader(Mol->vrmlfile, HB_COLOR_RED, HB_COLOR_GREEN, HB_COLOR_BLUE); // hbonds
	         break;
        case 8 : WireHeader(Mol->vrmlfile, SB_COLOR_RED, SB_COLOR_GREEN, SB_COLOR_BLUE); // sbonds
	         break;
	} // switch
      if ( (bstream[v]) && (bstream[v].pcount() > 0) )
        for (int ostr = 0; ostr < bstream[v].pcount(); ostr++)
          gzprintf(Mol->vrmlfile, "%c", bstream[v].str()[ostr]);
      gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tcoordIndex [\n");
      bsi = 0;
      for ( ; coordiptr[v] > 0; coordiptr[v]--) {
        gzprintf(Mol->vrmlfile, "\t\t\t%d", bsi);
        bsi++;
        gzprintf(Mol->vrmlfile, " , %d , -1,\n", bsi);
        bsi++;
        }
      gzprintf(Mol->vrmlfile, "\t\t]\n\t}\n}\n");
      }
  }
}

// traceSurface does not work yet. Help appreciated.
void pdb2vrml::traceSurface(void) {
  unsigned int posA;
  int v;
  int defC = 0;
  int defN = 0;
  int defH = 0;
  int defO = 0;
  int defS = 0;
  int defP = 0;
  int defHoldUp = 0;
  
  int carbonstart, carboncount;
  int nitrogenstart, nitrogencount;
  int oxygenstart, oxygencount;
  int sulfurstart, sulfurcount;
  int phosphatestart, phosphatecount;
  int hydrogenstart, hydrogencount;
  
  ofstream forSurf("tmp4surf");
  if ( (!forSurf) || (!forSurf.is_open()) ) return;
  
  carbonstart = 0;
  carboncount = 0;
  for (posA = 0; posA < Mol->TotAtoms; posA++)
    if (Mol->AtomList[posA].name[1] == 'C') {
      forSurf << carboncount << " " << C_ATOM_RADIUS << " " << Mol->AtomList[posA].x << " " << Mol->AtomList[posA].y << " " << Mol->AtomList[posA].z << endl;
      carboncount++;
      }

  nitrogenstart = carboncount;
  nitrogencount = nitrogenstart;
  for (posA = 0; posA < Mol->TotAtoms; posA++)
    if (Mol->AtomList[posA].name[1] == 'N') {
      forSurf << nitrogencount << " " << N_ATOM_RADIUS << " " << Mol->AtomList[posA].x << " " << Mol->AtomList[posA].y << " " << Mol->AtomList[posA].z << endl;
      nitrogencount++;
      }

  hydrogenstart = nitrogencount;
  hydrogencount = hydrogenstart;
  for (posA = 0; posA < Mol->TotAtoms; posA++)
    if (Mol->AtomList[posA].name[1] == 'H') {
      forSurf << hydrogencount << " " << H_ATOM_RADIUS << " " << Mol->AtomList[posA].x << " " << Mol->AtomList[posA].y << " " << Mol->AtomList[posA].z << endl;
      hydrogencount++;
      }

  oxygenstart = hydrogencount;
  oxygencount = oxygenstart;
  for (posA = 0; posA < Mol->TotAtoms; posA++)
    if (Mol->AtomList[posA].name[1] == 'O') {
      forSurf << oxygencount << " " << O_ATOM_RADIUS << " " << Mol->AtomList[posA].x << " " << Mol->AtomList[posA].y << " " << Mol->AtomList[posA].z << endl;
      oxygencount++;
      }

  sulfurstart = oxygencount;
  sulfurcount = sulfurstart;
  for (posA = 0; posA < Mol->TotAtoms; posA++)
    if (Mol->AtomList[posA].name[1] == 'S') {
      forSurf << sulfurcount << " " << S_ATOM_RADIUS << " " << Mol->AtomList[posA].x << " " << Mol->AtomList[posA].y << " " << Mol->AtomList[posA].z << endl;
      sulfurcount++;
      }

  phosphatestart = sulfurcount;
  phosphatecount = phosphatestart;
  for (posA = 0; posA < Mol->TotAtoms; posA++)
    if (Mol->AtomList[posA].name[1] == 'P') {
      forSurf << phosphatecount << " " << P_ATOM_RADIUS << " " << Mol->AtomList[posA].x << " " << Mol->AtomList[posA].y << " " << Mol->AtomList[posA].z << endl;
      phosphatecount++;
      }

  forSurf.flush();
  forSurf.close();
  
  FILE *surfpipe;
  char pipeline[256];
  surfpipe = popen("./surf -W 1 tmp4surf", "r");
  while (!feof(surfpipe)) fgets(pipeline, 256, surfpipe); // hmm...
  pclose(surfpipe);
  
  unsigned int coordiptr = 0;
  ifstream fromSurf("tmp4surf.tri");
  if ( (!fromSurf) || (!fromSurf.is_open()) ) { 
    cout << "Surf error.\n";
    return;
    }
  float curcoord;
  int atomid;
  while (!fromSurf.eof()) {
    if (defHoldUp == 0) 
      fromSurf >> atomid; 
    else {
      defHoldUp = 0;
      }
  if ( (atomid >= carbonstart) && (atomid < carboncount+carbonstart) && (defC!= 2) ) {
        if (defC == 0) {
          SurfaceHeader(Mol->vrmlfile, "C", C_COLOR_RED, C_COLOR_GREEN, C_COLOR_BLUE);
          coordiptr = 0;
	  defC = 1;
	  }
      for (v = 1; v <= 3; v++) {
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "\t\t\t%f ", curcoord);
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "%f ", curcoord);
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "%f ,\n", curcoord);
        fromSurf >> curcoord;
        fromSurf >> curcoord;
        fromSurf >> curcoord;
	coordiptr++;
	}      
      } else
    if (defC == 1) {
      defHoldUp = 1;
        gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tcoordIndex [\n");
        v = 0;
        for ( ; coordiptr > 0; coordiptr--) {
          gzprintf(Mol->vrmlfile, "\t\t\t%d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d , -1,\n", v);
          v++;
          }
        gzprintf(Mol->vrmlfile, "\t\t]\n\t");
        gzprintf(Mol->vrmlfile, "\tsolid FALSE\n");
        gzprintf(Mol->vrmlfile, "\t\tnormal Normal {\n");
        gzprintf(Mol->vrmlfile, "\t\t\tvector [\n");
        int atomid2;
        ifstream fromSurf2("tmp4surf.tri");
        if ( (!fromSurf2) || (!fromSurf2.is_open()) ) return;
        fromSurf2 >> atomid2;
        while ( (atomid2 < carboncount) && (!fromSurf2.eof()) ) {
          for ( int v2 = 1;  v2 <= 3; v2++) {
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "\t\t\t%f ", curcoord);
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "%f ", curcoord);
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "%f ,\n", curcoord);
            coordiptr++;
            }      
          fromSurf2 >> atomid2;
	  }
        fromSurf2.close();
	gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tnormalIndex [\n");
        v = 0;
        for ( ; coordiptr > 0; coordiptr--) {
          gzprintf(Mol->vrmlfile, "\t\t\t%d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d , -1,\n", v);
          v++;
          }
        gzprintf(Mol->vrmlfile, "\t\t]\n\t}\n}\n");
      defC = 2;
    } else
    if ( (atomid >= nitrogenstart) && (atomid < nitrogencount) && (defN != 2) ) {
      if (defN == 0) {
        SurfaceHeader(Mol->vrmlfile, "N", N_COLOR_RED, N_COLOR_GREEN, N_COLOR_BLUE);
	coordiptr = 0;
	defN = 1;
	}
      for (v = 1; v <= 3; v++) {
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "\t\t\t%f ", curcoord);
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "%f ", curcoord);
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "%f ,\n", curcoord);
        fromSurf >> curcoord;
        fromSurf >> curcoord;
        fromSurf >> curcoord;
	coordiptr++;
	}      
      } else
    if (defN == 1) {
        defHoldUp = 1;
        gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tcoordIndex [\n");
        v = 0;
        for ( ; coordiptr > 0; coordiptr--) {
          gzprintf(Mol->vrmlfile, "\t\t\t%d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d , -1,\n", v);
          v++;
          }
        gzprintf(Mol->vrmlfile, "\t\t]\n\t");
        gzprintf(Mol->vrmlfile, "\tsolid FALSE\n");
        gzprintf(Mol->vrmlfile, "\t\tnormal Normal {\n");
        gzprintf(Mol->vrmlfile, "\t\t\tvector [\n");
        int atomid2;
        ifstream fromSurf2("tmp4surf.tri");
        if ( (!fromSurf2) || (!fromSurf2.is_open()) ) return;
        fromSurf2 >> atomid2;
        while ( (atomid2 < nitrogenstart) && (!fromSurf2.eof()) ) {
          for ( int v2 = 1;  v2 <= 3; v2++) {
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            }      
          fromSurf2 >> atomid2;
	  }
        while ( (atomid2 < nitrogencount) && (!fromSurf2.eof()) ) {
          for ( int v2 = 1;  v2 <= 3; v2++) {
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "\t\t\t%f ", curcoord);
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "%f ", curcoord);
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "%f ,\n", curcoord);
            coordiptr++;
            }      
          fromSurf2 >> atomid2;
	  }
        fromSurf2.close();
	gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tnormalIndex [\n");
        v = 0;
        for ( ; coordiptr > 0; coordiptr--) {
          gzprintf(Mol->vrmlfile, "\t\t\t%d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d , -1,\n", v);
          v++;
          }
        gzprintf(Mol->vrmlfile, "\t\t]\n\t}\n}\n");
      defN = 2;
      } else
    if ( (atomid >= hydrogenstart) && (atomid < hydrogencount) && (defH != 2) ) {
      if (defH == 0) {
        SurfaceHeader(Mol->vrmlfile, "H", H_COLOR_RED, H_COLOR_GREEN, H_COLOR_BLUE);
	coordiptr = 0;
	defH = 1;
	}
      for (v = 1; v <= 3; v++) {
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "\t\t\t%f ", curcoord);
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "%f ", curcoord);
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "%f ,\n", curcoord);
        fromSurf >> curcoord;
        fromSurf >> curcoord;
        fromSurf >> curcoord;
	coordiptr++;
	}      
      } else
    if (defH == 1) {
        defHoldUp = 1;
        gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tcoordIndex [\n");
        v = 0;
        for ( ; coordiptr > 0; coordiptr--) {
          gzprintf(Mol->vrmlfile, "\t\t\t%d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d , -1,\n", v);
          v++;
          }
        gzprintf(Mol->vrmlfile, "\t\t]\n\t");
        gzprintf(Mol->vrmlfile, "\tsolid FALSE\n");
        gzprintf(Mol->vrmlfile, "\t\tnormal Normal {\n");
        gzprintf(Mol->vrmlfile, "\t\t\tvector [\n");
        int atomid2;
        ifstream fromSurf2("tmp4surf.tri");
        if ( (!fromSurf2) || (!fromSurf2.is_open()) ) return;
        fromSurf2 >> atomid2;
        while ( (atomid2 < hydrogenstart) && (!fromSurf2.eof()) ) {
          for ( int v2 = 1;  v2 <= 3; v2++) {
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            }      
          fromSurf2 >> atomid2;
	  }
        while ( (atomid2 < hydrogencount) && (!fromSurf2.eof()) ) {
          for ( int v2 = 1;  v2 <= 3; v2++) {
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "\t\t\t%f ", curcoord);
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "%f ", curcoord);
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "%f ,\n", curcoord);
            coordiptr++;
            }      
          fromSurf2 >> atomid2;
	  }
        fromSurf2.close();
	gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tnormalIndex [\n");
        v = 0;
        for ( ; coordiptr > 0; coordiptr--) {
          gzprintf(Mol->vrmlfile, "\t\t\t%d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d , -1,\n", v);
          v++;
          }
        gzprintf(Mol->vrmlfile, "\t\t]\n\t}\n}\n");
      defH = 2;
      } else
    if ( (atomid >= oxygenstart) && (atomid < oxygencount) && (defO != 2) && (!fromSurf.eof()) ) {
      if (defO == 0) {
        SurfaceHeader(Mol->vrmlfile, "O", O_COLOR_RED, O_COLOR_GREEN, O_COLOR_BLUE);
	coordiptr = 0;
	defO = 1;
	}
      for (v = 1; v <= 3; v++) {
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "\t\t\t%f ", curcoord);
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "%f ", curcoord);
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "%f ,\n", curcoord);
        fromSurf >> curcoord;
        fromSurf >> curcoord;
        fromSurf >> curcoord;
	coordiptr++;
	}      
      } else
    if (defO == 1) {
        defHoldUp = 1;
        gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tcoordIndex [\n");
        v = 0;
        for ( ; coordiptr > 0; coordiptr--) {
          gzprintf(Mol->vrmlfile, "\t\t\t%d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d , -1,\n", v);
          v++;
          }
        gzprintf(Mol->vrmlfile, "\t\t]\n\t");
        gzprintf(Mol->vrmlfile, "\tsolid FALSE\n");
        gzprintf(Mol->vrmlfile, "\t\tnormal Normal {\n");
        gzprintf(Mol->vrmlfile, "\t\t\tvector [\n");
        int atomid2;
        ifstream fromSurf2("tmp4surf.tri");
        if ( (!fromSurf2) || (!fromSurf2.is_open()) ) return;
        fromSurf2 >> atomid2;
        while ( (atomid2 < oxygenstart) && (!fromSurf2.eof()) ) {
          for ( int v2 = 1;  v2 <= 3; v2++) {
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            }      
          fromSurf2 >> atomid2;
	  }
        while ( (atomid2 < oxygencount) && (!fromSurf2.eof()) ) {
          for ( int v2 = 1;  v2 <= 3; v2++) {
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "\t\t\t%f ", curcoord);
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "%f ", curcoord);
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "%f ,\n", curcoord);
            coordiptr++;
            }      
          fromSurf2 >> atomid2;
	  }
        fromSurf2.close();
	gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tnormalIndex [\n");
        v = 0;
        for ( ; coordiptr > 0; coordiptr--) {
          gzprintf(Mol->vrmlfile, "\t\t\t%d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d , -1,\n", v);
          v++;
          }
        gzprintf(Mol->vrmlfile, "\t\t]\n\t}\n}\n");
      defO = 2;
      } else
    if ( (atomid >= sulfurstart) && (atomid < sulfurcount) && (defS != 2) ) {
      if (defS == 0) {
        SurfaceHeader(Mol->vrmlfile, "S", S_COLOR_RED, S_COLOR_GREEN, S_COLOR_BLUE);
	coordiptr = 0;
	defS = 1;
	}
      for (v = 1; v <= 3; v++) {
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "\t\t\t%f ", curcoord);
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "%f ", curcoord);
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "%f ,\n", curcoord);
        fromSurf >> curcoord;
        fromSurf >> curcoord;
        fromSurf >> curcoord;
	coordiptr++;
	}      
      } else
    if (defS == 1) {
        defHoldUp = 1;
        gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tcoordIndex [\n");
        v = 0;
        for ( ; coordiptr > 0; coordiptr--) {
          gzprintf(Mol->vrmlfile, "\t\t\t%d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d , -1,\n", v);
          v++;
          }
        gzprintf(Mol->vrmlfile, "\t\t]\n\t");
        gzprintf(Mol->vrmlfile, "\tsolid FALSE\n");
        gzprintf(Mol->vrmlfile, "\t\tnormal Normal {\n");
        gzprintf(Mol->vrmlfile, "\t\t\tvector [\n");
        int atomid2;
        ifstream fromSurf2("tmp4surf.tri");
        if ( (!fromSurf2) || (!fromSurf2.is_open()) ) return;
        fromSurf2 >> atomid2;
        while ( (atomid2 < sulfurstart) && (!fromSurf2.eof()) ) {
          for ( int v2 = 1;  v2 <= 3; v2++) {
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            }      
          fromSurf2 >> atomid2;
	  }
        while ( (atomid2 < sulfurcount) && (!fromSurf2.eof()) ) {
          for ( int v2 = 1;  v2 <= 3; v2++) {
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "\t\t\t%f ", curcoord);
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "%f ", curcoord);
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "%f ,\n", curcoord);
            coordiptr++;
            }      
          fromSurf2 >> atomid2;
	  }
        fromSurf2.close();
	gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tnormalIndex [\n");
        v = 0;
        for ( ; coordiptr > 0; coordiptr--) {
          gzprintf(Mol->vrmlfile, "\t\t\t%d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d , -1,\n", v);
          v++;
          }
        gzprintf(Mol->vrmlfile, "\t\t]\n\t}\n}\n");
      defS = 2;
      } else
    if ( (atomid >= phosphatestart) && (atomid < phosphatecount) && (defP != 2) ) {
      if (defP == 0) {
        SurfaceHeader(Mol->vrmlfile, "P", P_COLOR_RED, P_COLOR_GREEN, P_COLOR_BLUE);
	coordiptr = 0;
	defP = 1;
	}
      for (v = 1; v <= 3; v++) {
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "\t\t\t%f ", curcoord);
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "%f ", curcoord);
        fromSurf >> curcoord;
        gzprintf(Mol->vrmlfile, "%f ,\n", curcoord);
        fromSurf >> curcoord;
        fromSurf >> curcoord;
        fromSurf >> curcoord;
	coordiptr++;
	}      
      } else
    if (defP == 1) {
        defHoldUp = 1;
        gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tcoordIndex [\n");
        v = 0;
        for ( ; coordiptr > 0; coordiptr--) {
          gzprintf(Mol->vrmlfile, "\t\t\t%d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d , -1,\n", v);
          v++;
          }
        gzprintf(Mol->vrmlfile, "\t\t]\n\t");
        gzprintf(Mol->vrmlfile, "\tsolid FALSE\n");
        gzprintf(Mol->vrmlfile, "\t\tnormal Normal {\n");
        gzprintf(Mol->vrmlfile, "\t\t\tvector [\n");
        int atomid2;
        ifstream fromSurf2("tmp4surf.tri");
        if ( (!fromSurf2) || (!fromSurf2.is_open()) ) return;
        fromSurf2 >> atomid2;
        while ( (atomid2 < phosphatestart) && (!fromSurf2.eof()) ) {
          for ( int v2 = 1;  v2 <= 3; v2++) {
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            }      
          fromSurf2 >> atomid2;
	  }
        while ( (atomid2 < phosphatecount) && (!fromSurf2.eof()) ) {
          for ( int v2 = 1;  v2 <= 3; v2++) {
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "\t\t\t%f ", curcoord);
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "%f ", curcoord);
            fromSurf2 >> curcoord;
            gzprintf(Mol->vrmlfile, "%f ,\n", curcoord);
            coordiptr++;
            }      
          fromSurf2 >> atomid2;
	  }
        fromSurf2.close();
	gzprintf(Mol->vrmlfile, "\t\t\t]\n\t\t}\n\t\tnormalIndex [\n");
        v = 0;
        for ( ; coordiptr > 0; coordiptr--) {
          gzprintf(Mol->vrmlfile, "\t\t\t%d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d", v);
          v++;
          gzprintf(Mol->vrmlfile, " , %d , -1,\n", v);
          v++;
          }
        gzprintf(Mol->vrmlfile, "\t\t]\n\t}\n}\n");
      defP = 2;
      } else {
        return; }
  } // while
}

int pdb2vrml::writevrml(int display) {
  gzprintf(Mol->vrmlfile,"#VRML V2.0 utf8\n");
  gzprintf(Mol->vrmlfile, "Viewpoint {\n");
  gzprintf(Mol->vrmlfile, "\tposition %f %f %f\n", (Mol->MinX+Mol->MaxX)/2, (Mol->MinY+Mol->MaxY)/2, Mol->MaxZ*2.5);
  gzprintf(Mol->vrmlfile, "\tdescription \"VRML representation of a PDB file.\"\n");
  gzprintf(Mol->vrmlfile, "}\n");
  if (bkScene == 1) {
    gzprintf(Mol->vrmlfile, "Background {\n");
    if (toptex) gzprintf(Mol->vrmlfile, "\ttopUrl \"%s\"", toptex);
    if (bottomtex) gzprintf(Mol->vrmlfile, "\tbottomUrl \"%s\"", bottomtex);
    if (fronttex) gzprintf(Mol->vrmlfile, "\tfrontUrl \"%s\"", fronttex);
    if (backtex) gzprintf(Mol->vrmlfile, "\tbackUrl \"%s\"", backtex);
    if (righttex) gzprintf(Mol->vrmlfile, "\trightUrl \"%s\"", righttex);
    if (lefttex) gzprintf(Mol->vrmlfile, "\tleftUrl \"%s\"", lefttex);
    gzprintf(Mol->vrmlfile, "\tskyColor [ %f %f %f, %f %f %f, %f %f %f, %f %f %f, %f %f %f, %f %f %f ]", SKY1_COLOR_RED, SKY1_COLOR_GREEN, SKY1_COLOR_BLUE, SKY2_COLOR_RED, SKY2_COLOR_GREEN, SKY2_COLOR_BLUE, SKY3_COLOR_RED, SKY3_COLOR_GREEN, SKY3_COLOR_BLUE, SKY4_COLOR_RED, SKY4_COLOR_GREEN, SKY4_COLOR_BLUE, SKY5_COLOR_RED, SKY5_COLOR_GREEN, SKY5_COLOR_BLUE, SKY6_COLOR_RED, SKY6_COLOR_GREEN, SKY6_COLOR_BLUE);
    gzprintf(Mol->vrmlfile, "\tskyAngle [ %f, %f, %f, %f, %f ]", SKY_ANGLE1, SKY_ANGLE2, SKY_ANGLE3, SKY_ANGLE4, SKY_ANGLE5);
    gzprintf(Mol->vrmlfile, "\tgroundColor [ %f %f %f, %f %f %f, %f %f %f ]", GROUND1_COLOR_RED, GROUND1_COLOR_GREEN, GROUND1_COLOR_BLUE, GROUND2_COLOR_RED, GROUND2_COLOR_GREEN, GROUND2_COLOR_BLUE, GROUND3_COLOR_RED, GROUND3_COLOR_GREEN, GROUND3_COLOR_BLUE);
    gzprintf(Mol->vrmlfile, "\tgroundAngle [ %f, %f ]", GROUND_ANGLE1, GROUND_ANGLE2);
    gzprintf(Mol->vrmlfile, "}\n");
    }
  switch (display) {
    case DISPLAY_SPACEFILL : traceSpaceFill();
                             break;
    case DISPLAY_BALLSTICK : traceBallStick();
                             break;
    case DISPLAY_WIREFRAME : traceWire();
                             break;
    case DISPLAY_SURFACE   : traceSurface();
                             break;
    } // switch
 return 0;
}




void pdb2vrml::traceBallStick(void) {
  unsigned char defN=0,defO=0,defC=0,defH=0,defS=0,defP=0;
  int itmp; // float mtmp, newX, newY, newZ, Zvect;
  for (unsigned int i=1-1; i<=Mol->TotAtoms-1; i++) {
    gzprintf(Mol->vrmlfile, "Transform {\n\tchildren [\n");
    switch (Mol->AtomList[i].name[1]) {
      case 'C' : if (defC==1) gzprintf(Mol->vrmlfile, "\t\tUSE C\n");
                 else {
		       traceSphere(Mol->vrmlfile, "C", C_COLOR_RED, C_COLOR_GREEN, C_COLOR_BLUE, "0.400000");
		       defC=1;
                      }
                 break;
      case 'N' : if (defN==1) gzprintf(Mol->vrmlfile, "\t\tUSE N\n");
                 else {
		       traceSphere(Mol->vrmlfile, "N", N_COLOR_RED, N_COLOR_GREEN, N_COLOR_BLUE, "0.400000");
                       defN=1;
                      }
                 break;
      case 'O' : if (defO==1) gzprintf(Mol->vrmlfile, "\t\tUSE O\n");
                 else {
		       traceSphere(Mol->vrmlfile, "O", O_COLOR_RED, O_COLOR_GREEN, O_COLOR_BLUE, "0.400000");
                       defO=1;
                      }
                 break;
      case 'S' : if (defS==1) gzprintf(Mol->vrmlfile, "\t\tUSE S\n");
                 else {
		       traceSphere(Mol->vrmlfile, "S", S_COLOR_RED, S_COLOR_GREEN, S_COLOR_BLUE, "0.400000");
                       defS=1;
                      }
                 break;
      case 'H' : if (defH==1) gzprintf(Mol->vrmlfile, "\t\tUSE H\n");
                 else {
		       traceSphere(Mol->vrmlfile, "H", H_COLOR_RED, H_COLOR_GREEN, H_COLOR_BLUE, "0.400000");
                       defH=1;
                      }
                 break;
      case 'P' : if (defP==1) gzprintf(Mol->vrmlfile, "\t\tUSE P\n");
                 else {
		       traceSphere(Mol->vrmlfile, "P", P_COLOR_RED, P_COLOR_GREEN, P_COLOR_BLUE, "0.400000");
                       defP=1;
                      }
                 break;
      } // switch
    gzprintf(Mol->vrmlfile, "\t]\n\ttranslation\t%f %f %f\n}\n", Mol->AtomList[i].x, Mol->AtomList[i].y, Mol->AtomList[i].z);
    } // for loop (balls)
  for (unsigned int i=1-1; i<=Mol->TotAtoms-1; i++) {
    for (int j=0; j<=3; j++) { // No bkbonds here...
	 itmp = 0;
	 while ((Mol->AtomList[i].bonds[j][itmp] > 0) && (itmp<MAXBONDS)) 
               {
	       if (j==0)
                  traceBond(Mol->vrmlfile,i,Mol->AtomList[i].bonds[j][itmp],NB_COLOR_RED, NB_COLOR_GREEN, NB_COLOR_BLUE,0,1);
	       if (j==1) 
                  traceBond(Mol->vrmlfile,i,Mol->AtomList[i].bonds[j][itmp],CB_COLOR_RED, CB_COLOR_GREEN, CB_COLOR_BLUE,0,1);
	       if (j==2) 
                  traceBond(Mol->vrmlfile,i,Mol->AtomList[i].bonds[j][itmp],HB_COLOR_RED, HB_COLOR_GREEN, HB_COLOR_BLUE,0,1);
	       if (j==3) 
                  traceBond(Mol->vrmlfile,i,Mol->AtomList[i].bonds[j][itmp],SB_COLOR_RED, SB_COLOR_GREEN, SB_COLOR_BLUE,0,1);
	       itmp++;
               } // end of while (found a bond)
    }
  } // second for loop (sticks)
}


void pdb2vrml::traceSpaceFill(void) {
  unsigned char defN=0,defO=0,defC=0,defH=0,defS=0,defP=0;
  for (unsigned int i=1-1; i<=Mol->TotAtoms-1; i++) {
    gzprintf(Mol->vrmlfile, "Transform {\n\tchildren [\n");
    switch (Mol->AtomList[i].name[1]) {
      case 'C' : if (defC==1) gzprintf(Mol->vrmlfile, "\t\tUSE C\n");
                 else {
		       traceSphere(Mol->vrmlfile, "C", C_COLOR_RED, C_COLOR_GREEN, C_COLOR_BLUE, "0.910000");
                       defC=1;
                      }
                 break;
      case 'N' : if (defN==1) gzprintf(Mol->vrmlfile, "\t\tUSE N\n");
                 else {
		       traceSphere(Mol->vrmlfile, "N", N_COLOR_RED, N_COLOR_GREEN, N_COLOR_BLUE, "0.750000");
                       defN=1;
                      }
                 break;
      case 'O' : if (defO==1) gzprintf(Mol->vrmlfile, "\t\tUSE O\n");
                 else {
		       traceSphere(Mol->vrmlfile, "O", O_COLOR_RED, O_COLOR_GREEN, O_COLOR_BLUE, "0.650000");
                       defO=1;
                      }
                 break;
      case 'S' : if (defS==1) gzprintf(Mol->vrmlfile, "\t\tUSE S\n");
                 else {
		       traceSphere(Mol->vrmlfile, "S", S_COLOR_RED, S_COLOR_GREEN, S_COLOR_BLUE, "1.090000");
                       defS=1;
                      }
                 break;
      case 'H' : if (defH==1) gzprintf(Mol->vrmlfile, "\t\tUSE H\n");
                 else {
		       traceSphere(Mol->vrmlfile, "H", H_COLOR_RED, H_COLOR_GREEN, H_COLOR_BLUE, "0.790000");
                       defH=1;
                      }
                 break;
      case 'P' : if (defP==1) gzprintf(Mol->vrmlfile, "\t\tUSE P\n");
                 else {
		       traceSphere(Mol->vrmlfile, "P", P_COLOR_RED, P_COLOR_GREEN, P_COLOR_BLUE, "1.230000");
                       defP=1;
                      }
                 break;
      } // switch
    gzprintf(Mol->vrmlfile, "\t]\n\ttranslation\t%f %f %f\n}\n", Mol->AtomList[i].x, Mol->AtomList[i].y, Mol->AtomList[i].z);
    } // for loop
}

